Rem
	This is the physics body type.  With physics bodies, you can move
	the phyisics body or give it a velocity (rotational or linear) in one command.
EndRem

Private	'define some private vars for pbody type
	Global pbodylist:TList = New TList
Public


Type pbody	'physics body type
	Field x:Float	'used for rotation around center etc
	Field y:Float	'
	Field vlist:TList	'stores all verlets associated with the physics body
	Field weighted:Int	'if true then (x,y) will be based on weighted average rather than unweighted average
	Field children:TList	'stores its children
	Field setremove:Int	'tells if the pbody is set to be removed
	
	'''''''''''''''''''''''''''''
	'Functions'''''''''''''''''''
	'''''''''''''''''''''''''''''
	Function Create:pbody(x:Float,y:Float,weighted:Int = True)	'create pbody
		Local p:pbody = New pbody		'create a new pbody
		p.x = x					'define x and y values
		p.y = y
		p.vlist = New TList			'make a new list for verlets to be stored in
		p.children = New TList			'make a list of the children
		p.weighted = weighted			'define weighted value
		pbodylist.addlast(p:pbody)
		Return p:pbody				'return the pbody
	End Function
	
	Function update()	'update all physics bodies
		Local p:pbody	'physicsbody object
		Local v:verlet	'verlet object
		Local avgx:Float	'x used to average the x,y values
		Local avgy:Float	'y used to average the x,y values
		Local divd:Float	'divide avgx and avgy by this at the end to get the average
		Local c:pbody	'used to loop through children of said physics body
		
		For p:pbody = EachIn pbodylist	'loop through all pbody(s)
			avgx = 0				'reset avgx and avgy
			avgy = 0
			divd = 0				'reset divd var
			For v:verlet = EachIn p.vlist	'loop through all verlets in that pbody
				If p.weighted Then		'if it is a weighted average
					avgx :+v.x*v.mass			'do weighted average formula
					avgy :+v.y*v.mass
					divd :+v.mass
				Else
					divd :+1			'else do the normal averageing formula
					avgx :+v.x
					avgy :+v.y
				EndIf
			Next
			p.x = avgx/divd	'finish off the formula to average them
			p.y = avgy/divd
			For c:pbody = EachIn p.children	'lets gc the pbody's children if they have been deleted
				If c = Null Then
					p.children.remove(c:pbody)
				Else
					If c.setremove = True Then
						p.children.remove(c:pbody)
					EndIf
				EndIf
			Next
		Next
		
	End Function
	
	'''''''''''''''''''''''
	'Methods'''''''''''''''
	'''''''''''''''''''''''
	Method AddChild(p:pbody)		'add a child physics body (note: children are not physically attached in any way. They are simply associated with eachother)
		children.addlast(p:pbody)
	End Method
	
	Method AddVerlet(v:verlet)		'adds a verlet to the vlist of the specified pbody
		vlist.addlast(v:verlet)
	End Method
	
	Method RemoveVerlet(v:verlet)	'removes a verlet from the vlist of the specified pbody
		vlist.remove(v:verlet)
	End Method
	
	Method Move(xx:Float,yy:Float)	'Moves the whole pbody
		Local v:verlet
		For v:verlet = EachIn vlist		'loop through each verlet in the vlist
			v.move(xx,yy)
		Next
		For Local c:pbody = EachIn children
			c.move(xx,yy)
		Next
		x:+xx
		y:+yy
	End Method
	
	Method Position(xx:Float,yy:Float)	'Moves the whole pbody
		Local v:verlet
		For v:verlet = EachIn vlist		'loop through each verlet in the vlist
			v.position(xx,yy)
		Next
		For Local c:pbody = EachIn children
			c.position(xx,yy)
		Next
		x = xx
		y = yy
	End Method
	
	Method AddVelocity(xx:Float,yy:Float)	'adds a velocity to the pbody
		Local v:verlet
		For v:verlet = EachIn vlist
			v.addvelocity(xx,yy)
		Next
		For Local c:pbody = EachIn children
			c.addvelocity(xx,yy)
		Next
	End Method
	
	Method GetXVelocity:Float()
		Local v:verlet
		Local xx:Float
		Local divd:Float
		For v:verlet = EachIn vlist
			xx = (v.x-v.ox)*v.mass
			divd = divd + v.mass
		Next
		Return (xx/divd)
	End Method
	
	Method GetYVelocity:Float()
		Local v:verlet
		Local yy:Float
		Local divd:Float
		For v:verlet = EachIn vlist
			yy = (v.y-v.oy)*v.mass
			divd = divd + v.mass
		Next
		Return (yy/divd)
	End Method
	
	Method Rotate(angle:Float)
		Local v:verlet
		Local tary:Float[2]	'used to get x an y values
		For v:verlet = EachIn vlist
			tary = rpoint(x,y,v.x,v.y,angle)
			v.position(tary[0],tary[1])
		Next
		For Local c:pbody = EachIn children
			c.rotate(angle)
		Next
		
	End Method
	
	Method AddRotationalVelocity(angle:Float)
		Local v:verlet
		Local tary:Float[2]	'used to get x an y values
		For v:verlet = EachIn vlist
			tary = rpoint(x,y,v.x,v.y,angle)
			v.addvelocity((tary[0])-v.x,(tary[1])-v.y)
		Next
		For Local c:pbody = EachIn children
			c.addrotationalvelocity(angle)
		Next
	End Method
	
	Method SetRotationalVelocity(angle:Float)
		Local v:verlet
		Local tary:Float[2]	'used to get x an y values
		For v:verlet = EachIn vlist
			tary = rpoint(x,y,v.x,v.y,angle)
			v.SetVelocity((tary[0])-v.x,(tary[1])-v.y)
		Next
		For Local c:pbody = EachIn children
			c.setrotationalvelocity(angle)
		Next
		
	End Method
	
	Method remove(m:Int = 0)	'm = method of removal 0 = just pbody, 1 = self and all verlets, 2 = self and all verlets and all children and their verlets
		If m > 0 Then
			If m = 2 Then
				For Local c:pbody = EachIn children
					c.remove(2)
				Next
			EndIf
			For Local v:verlet = EachIn vlist
				v.remove
			Next
		EndIf
		pbodylist.remove(Self)
		setremove = True
	End Method
	
End Type


Private
Function rpoint:Float[](x#,y#,x2#,y2#,ang#)
	Local a1:Float = ATan2((y2-y),(x2-x))
	ang = ang + a1
	Local dist:Float = Sqr((x-x2)^2 + (y-y2)^2)
	Local xx:Float = Cos(ang)*dist
	Local yy:Float = Sin(ang)*dist
	xx :+x
	yy :+y
	Return [xx#,yy#]
End Function
Public
